/***************************************************************************
*                        Probality Module
*
*   File    : pb_probability.h
*   Purpose : Header file for probability computing functions 
*             needed for solving the correlation problem
*   Author  : Azhar Desai
*   Date    : September, 2011
*
****************************************************************************/
#ifndef PB_PROBABILITY_H
#define PB_PROBABILITY_H

/***************************************************************************
*                             INCLUDED FILES
***************************************************************************/
#include <math.h>

/***************************************************************************
*                              MACROS & CONTSTANTS
***************************************************************************/

/*  Floats Comparisons */
#define EPSILON 1.0e-9
#define dbl_equals(a, b) (fabs((a)-(b)) < EPSILON)
#define dbl_gt(a, b) (a - b > EPSILON)

/* Asserting that probabilities are in range [0,1] */
#define ASSERT_PROB(p) assert(p >= 0-EPSILON && p <= 1+EPSILON)

/* TODO constants below should be prefixed with module abbreviation */
/* Constants for controlling pb_ps  */
enum PSopt {Z_TRUE, Z_FALSE, Z_SUM};
typedef enum PSopt PSopt_t;

/* Constants for controlling pb_calc_IU */
enum IUopt {I_OPT, U_OPT};
typedef enum IUopt IUopt_t;

/***************************************************************************
*                               PROTOTYPES
***************************************************************************/

/* Calculate m */
unsigned int  pb_calc_m (unsigned int  N, 
			 unsigned int k, 
			 unsigned int t);

/* Calculate s: probability that y_i = b_i in the model for constant p */
double pb_calc_s (double p, unsigned int t);

/* Calculate s: with variable p */
double pb_calc_s_var_p(double *p,
		      unsigned int *taps,
		      unsigned int t,
		      unsigned int excl_t);

/* Caculate a expression that is used often */
double pb_ps (double p, 
	      double s, 
	      unsigned int m, 
	      unsigned int i, 
	      PSopt_t opt);

/* Update probability that z=a given new information h relations satisfied */
double pb_update(double p, 
		 double s,
		 unsigned int m,
		 unsigned int h);

/* Update probability that z=a in the case of variable p */
double pb_update_var_p(double p,
		       double *svals);

/* Calculate either I or U depending opt passed */
double pb_calc_IU(double p, 
		  double s, 
		  unsigned int m, 
		  unsigned int h,
		  IUopt_t opt);

#endif
