/***************************************************************************
*                        Computing Parity Checks
*
*   File    : pc_paritycheck.h
*   Purpose : Header file for library that computing parity check equations
*   Author  : Azhar Desai
*   Date    : September, 2011
*
****************************************************************************/
#ifndef PAR_CHECK_H
#define PAR_CHECK_H

/***************************************************************************
*                             INCLUDED FILES
***************************************************************************/
#include "bitarray.h"

/***************************************************************************
*                              MACROS & CONTSTANTS
***************************************************************************/
#define PC_SATISFIED_PROD 0
#define PC_SATISFIED_PROD_COMPLEMENT 1
#define PC_UNSATISFIED_PROD 2
#define PC_UNSATISFIED_PROD_COMPLEMENT 3
#define PC_NUM_PRODS 4

/***************************************************************************
*                            TYPE DEFINITIONS
***************************************************************************/
/* incomplete type to hide implementation */
struct pc_paritychecks_t;
typedef struct pc_paritychecks_t pc_paritychecks_t;

/***************************************************************************
*                               PROTOTYPES
***************************************************************************/

/* Create and destroy functions */
pc_paritychecks_t *pc_create(unsigned int N, 
			     unsigned int t,
			     unsigned int taps []);

void pc_destroy(pc_paritychecks_t *pchecks);

/* Access member variables num_polys, N and individual polynomials */
unsigned int pc_numpolys(pc_paritychecks_t *pchecks);
unsigned int pc_N(pc_paritychecks_t *pchecks);
unsigned int **pc_polys(pc_paritychecks_t *pchecks);

/* Compute the number of parity checks, h,  satisfied for each bit*/
void pc_h(pc_paritychecks_t *parchecks,
	  bit_array_t *z,
	  unsigned int h []);

/* Compute parity checks and the s_i values needed to update probability */
void pc_parcheck_s(pc_paritychecks_t *parchecks,
		   bit_array_t *z,
		   double p [],
		   double **s);

/* Check if a given sequence is produced by the specified LFSR */
unsigned char pc_check_lfsr(pc_paritychecks_t *parchecks, bit_array_t *z);

#endif
