#include <stdlib.h>
#include <check.h>

#include "../src/algorithm_b.h"
#include "../src/pb_probability.h"

/* Test that algorithm B runs without failing*/
START_TEST (test_algoB)
{
  unsigned int N = 28;
  double p = 0.75;
  unsigned int k = 3;
  unsigned int t = 2;
  /* 1 + X^2 + X^3 */
  unsigned int taps [] = {2,3};
  
  /* Create keystream */
  bit_array_t *z = BitArrayCreate(28);
  BitArraySetAll(z);
  unsigned int unsetbits [] = {0,5,11,14, 21, 25, 26}; 
  for (int i = 0; i < 7; i++)
    {
      BitArrayClearBit(z, unsetbits[i]);
    }
  
  /* LFSR sequence */
  bit_array_t *a = NULL;

  algorithm_b(p, k, t, taps, N, z, a);

}
END_TEST

/* Test  h_max */
START_TEST (test_h_max)
{
  /* Fixed answer, haven't verfied by hand */
  unsigned int ans = 8;
  unsigned int computed = calc_h_max(0.75, 0.6, 20);
  fail_unless(computed == ans,
	      "H_max computed as %u when it should be %u",
	      computed,
	      ans);

}
END_TEST


Suite *
algorithm_b_suite(void)
{
  Suite *s = suite_create("Algorithm B");
  
  TCase *tc_algoB = tcase_create("Test AlgoB");
  tcase_add_test(tc_algoB, test_algoB);
  suite_add_tcase(s, tc_algoB);
  
  TCase *tc_hmax = tcase_create("Compute H_max");
  tcase_add_test(tc_hmax, test_h_max);
  suite_add_tcase(s, tc_hmax);
  
  return s;
}


int 
main (void)
{
  int number_failed;
  Suite *s = algorithm_b_suite();
  SRunner *sr = srunner_create(s);

  /* For debugging test results */
  srunner_set_fork_status (sr, CK_NOFORK);

  srunner_run_all(sr, CK_NORMAL);
  number_failed = srunner_ntests_failed(sr);
  srunner_free(sr);
  return (number_failed == 0) ? EXIT_SUCCESS : EXIT_FAILURE;
}
