#!/usr/bin/python

# 
# Generate test data for the Fast Correlation Attack implemenation
# using a Geffer Combination generator
#
# Usage: ./combgen.py filename
# If filename is not specified defaults to test
# 
# The correlated output sequence is printed into filename.ans
#
# Azhar Desai

import sys
from itertools import izip

numlfsr = 3
global geffedata
global corrlfsr

def main(f, f_ans): 
    global geffedata, corrlfsr
    
    print welcomemsg
    print initstatemsg
    print tapsmsg

    N  = int(input("Please enter the number of keystream bits to generate: "))
    geffedata = [0.75, str(N)]
    corrlfsr = ""
    geffeseq = ""
    
    for (a, b, c) in izip(lfsr(corr=True), lfsr(), lfsr()):
        geffeseq += str(geffe(a,b,c))
        #print (a, b, c)
        N -= 1
        if N == 0: 
            break

    geffedata.append(geffeseq)
    
    # print out data to files
    print >> f, "\n".join(str(i) for i in geffedata)
    print >> f_ans, corrlfsr

def lfsr(corr=False):
    """ Get LFSR parameters and then yield clocked bits one by one """
    global geffedata, corrlfsr
    
    print "-----------------------------------------------------"
    print "                    LFSR                             "
    print "-----------------------------------------------------"
    state = [ 1 if x == '1' else 0 for x in raw_input("State: ") ]
    l = len(state)
    taps = raw_input("Taps: ").split()
    
    # store correlated lfsr in variable to print out to file
    if corr:
        geffedata.append(l)
        geffedata.append(len(taps))
        for t in taps:
            geffedata.append(t)
        corrlfsr = "".join(str(s) for s in state)
                
    # flip around feedback poly to work out
    taps = [l - int(i) for i in taps]
    
    #First yield initial state
    for bit in state:
        yield bit

    #Yield extra bits
    while(True):
        nextbit = 0
        for i in taps:
            nextbit = xor(nextbit, state[i])
        
        state.append(nextbit)
        state.pop(0)

        assert(len(state) == l)
        # append lfsr bit to ans file
        if corr:
            corrlfsr += str(nextbit)
        yield nextbit

def geffe(a, b, c):
    return xor(xor(a and b, b and c), c)

def xor(x, y):
    return 1 if x != y else 0

# For the kids
def testxor():
    assert(xor(1,0))
    assert(xor(0,1))
    assert(not xor(1,1))
    assert(not xor(0,0))
    
def testgeff():
    assert(not geffe(0,0,0))
    assert(geffe(0,0,1))
    assert(not geffe(0,1,0))
    assert(not geffe(0,1,1))
    assert(not geffe(1,0,0))
    assert(geffe(1,0,1))
    assert(geffe(1,1,0))
    assert(geffe(1,1,1))

welcomemsg="""
-----------------------------------------------------
               Geffe Generator                       
                                                         
     Enter the LFSR in order a, b, c where              
              g(a,b,c) = ab + bc + c                  
 -----------------------------------------------------

"""
tapsmsg = """
Enter the taps of the equations
Eg. the LFSR with the feedback polynomial 1 + X^2 + X^3"
would be represented as: 2 3

Taps: """
initstatemsg = """
Enter the initial state of the LFSR in binary in the
form: a_0 a_1 a_2 a_3 ... a_n. Eg.
State: 10101011
"""

    
if __name__=="__main__":
    filename = "test"
    if len(sys.argv) >= 2:
        filename = sys.argv[1]

    f = open(filename + ".test", 'w')
    f_ans = open(filename + ".ans", 'w')
    
    main(f, f_ans)

    f.close()
    f_ans.close()


